/*
 * TiVo initialization and setup code
 *
 * Copyright (C) 2000-2005 TiVo Inc. All Rights Reserved.
 */

#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/ide.h>
#include <linux/bootmem.h>
#include <linux/tivoconfig.h>
#include <asm/bootinfo.h>
#include <asm/mipsregs.h>
#include <asm/reboot.h>
#include <asm/traps.h>
#include <asm/page.h>
#include <asm/delay.h>
#include <asm/bcm7401.h>

#ifdef CONFIG_MIPS_BRCM974XX
#include "cfe_api.h"
#include "cfe_api_int.h"
#endif

extern void tivo_restart(char *command);
extern void tivo_halt(void);
extern void tivo_power_off(void);
extern void dump_tlb_all(void);

extern int panic_timeout;
extern struct ide_ops bcm_ide_ops;
extern int _end;

#ifdef CONFIG_TIVO_DEVEL
extern void uart_init( unsigned long uClkSpeed );
extern void uart_puts( const char *strng );
#endif

char arcs_cmdline[CL_SIZE];
#define MTN_RB_RAC_PREFETCH_LIMIT       __preg32(0xB0000408)

static void gen06_master_reset(void)
{
	*(volatile u32*)(BCM_SUN_TOP_CTRL_RESET_CTRL) |= 8;         // enable master reset
	*(volatile u32*)(BCM_SUN_TOP_CTRL_SW_RESET) |= 0x80000000;  // yank it!
    
    mdelay(500);
    printk("Master reset failed!");
}

static void config5_reset(void)
{
  	__asm__ __volatile__(
         	"   .set    noreorder\n"
  	       	"   .set    noat\n"
                  "   .word   0x4001b005\n"
                  "   nop\n"
                  "   ori     $1,3\n"
                  "   .word   0x4081b005\n"
  	       	"   .set    at\n"
  	       	"   .set    reorder\n"
  		: : : "$1", "memory" );
  
  	mdelay(500);
}

static void gen06_reboot(void)
{
	cli();
	gen06_master_reset();	// P0 reset, board reset
	config5_reset();	    // PX reset. bcm7401 reset
}


static void __init gen06_init_tlb( void )
{
    unsigned long lo0, lo1, hi;
    hi = 0xf0000000;
    lo0 = (((hi>>(4+2))&0x3fffffc0)|0x17);
    lo1 = ((((hi+OFFSET_16MBYTES)>>(4+2))&0x3fffffc0)|0x17);
    add_wired_entry(lo0, lo1, hi, PM_16M);
}

int turn_off_dma = 0;
static __init void configure_sata_pci_controller(void)
{
	*(volatile u32*)(BCM_SUN_TOP_CTRL_SATA_CTRL) = 0x4;
	*(volatile u32*)(PCI_BRIDGE_PCI_CTRL) = 0x33;
	/*Enabling byte swapping*/
	*(volatile u32*)(PCI_BRIDGE_SLV_MEM_WIN_BASE) = 0x02;
	/*Enabling byte swapping*/
	*(volatile u32*)(PCI_BRIDGE_CPU_TO_SATA_MEM_WIN_BASE) = CPU2PCI_PCI_SATA_PHYS_MEM_WIN0_BASE;
	/*Enabling byte swapping*/
	*(volatile u32*)(PCI_BRIDGE_CPU_TO_SATA_IO_WIN_BASE) = 0x02;
	/*FIXME:shmoo has to do this.*/
	*(volatile u32*)(0xb01061E8) = 0x40;
	if ( ! *(volatile u32*)(BCM_SUN_TOP_CTRL_STRAP_VALUE) & 0x100)
	{
	    /*We dont have 128MB strapped*/
	    turn_off_dma = 1;
	}
}

void __init tivo_setup(void)
{
	u32 config;
    
	gen06_init_tlb();

	*BCM_PCI_CMD_SELF   |= PCI_BUS_MASTER|PCI_IO_ENABLE|PCI_MEM_ENABLE;
	barrier();

	*BCM_PCI_GISB_BASE   = PCI_7401_PHYS_GISB_WIN_BASE;
	barrier();

	*BCM_PCI_MEM_W0_BASE = PCI_7401_PHYS_MEM_WIN_BASE;
	barrier();

	config = *BCM_PCI_SDRAM_ENDIAN_CTRL;
	config &= ~0x3;
	config |= 0x2;
	*BCM_PCI_SDRAM_ENDIAN_CTRL = config;
	barrier();
	
	*BCM_PCI_MEM_REMAP_W0 = CPU2PCI_CPU_PHYS_MEM_WIN_BASE;
	barrier();

	*BCM_PCI_MEM_REMAP_W1 = CPU2PCI_CPU_PHYS_MEM_WIN1_BASE;
	barrier();

	*BCM_PCI_MEM_REMAP_W2 = CPU2PCI_CPU_PHYS_MEM_WIN2_BASE;
	barrier();

	*BCM_PCI_MEM_REMAP_W3 = CPU2PCI_CPU_PHYS_MEM_WIN3_BASE;
	barrier();

	*BCM_PCI_IO_REMAP_W0  = CPU2PCI_CPU_PHYS_IO_WIN0_BASE;
	barrier();

	*BCM_PCI_IO_REMAP_W1  = CPU2PCI_CPU_PHYS_IO_WIN1_BASE;
	barrier();

	*BCM_PCI_IO_REMAP_W2  = CPU2PCI_CPU_PHYS_IO_WIN2_BASE;
	barrier();

#ifdef CONFIG_BLK_DEV_IDE
	ide_ops = &bcm_ide_ops;
#endif
	// uh huh... sure, it's const...
	*(unsigned long *)&mips_io_port_base = BCM_PCIIO_BASE;
	_machine_restart = tivo_restart;
	_machine_halt = tivo_halt;
	_machine_power_off = tivo_power_off;
	panic_timeout = 1;

	configure_sata_pci_controller();
}

// Note the arguments are listed here to match up with what init_arch()
// thinks it passes.  Really the arguments come from the bootprom, and
// only the first argument is valid, as a pointer to a single
// NULL-terminated string
//
#ifdef CONFIG_MIPS_BRCM974XX
int __init prom_init(int argc, char **argv, char **envp, int *prom_vec)
{
	unsigned long mem_end, mem_start = 0;
	TivoConfigValue brev, brid;
	extern void tivo_generic_setup(char *, int);
    extern void (*tivo_board_reboot)(void);
	char *brev_param;

	uint64_t cfe_ept, cfe_handle;
	unsigned int cfe_eptseal;
    char macaddr[32];

    memset(macaddr,0,32);

	// mips-linux expects FR bit cleared, this also makes sure the other
	// bits are in a sane state, but leave BEV on until we get our
	// exception handler installed, so that PROM gets any (unexpected)
	// exceptions
	write_32bit_cp0_register(CP0_STATUS, ST0_CU3 | ST0_BEV);
	write_32bit_cp0_register(CP0_WATCHLO, 0);
	write_32bit_cp0_register(CP0_WATCHHI, 0);

	mips_machgroup = MACH_GROUP_TIVO;

	// set this up now, in case of early panic
	tivo_board_reboot = gen06_reboot;

#ifdef CONFIG_TIVO_DEVEL
        uart_init( XTALFREQ );
        uart_puts( "Uarts Initialized.\n" );
#endif

    brev = kTivoConfigBoardIDFusionPX;

    brev_param = strstr(arcs_cmdline, "brev=");
    if (brev_param) {
        brev = simple_strtoul(brev_param + 5, NULL, 0) << 8;
    }
    brid = 0xff; 

	InitTivoConfig( brev );
	SetTivoConfig( kTivoConfigBoardID, brev | brid );

	if( GetTivoConfig( kTivoConfigMemSize, &mem_end ) != 0 )
	{
	    printk( "Couldn't get the memory size configuration\n" );
	    // Use a reasonable default of 128MB
	    mem_end = 128 * 1024 * 1024;
	}

	/*
	 * Check if a loader was used; if NOT, the 4 arguments are
	 * what CFE gives us (handle, 0, EPT and EPTSEAL)
	 */
	if (argc < 0) {
		cfe_handle = (uint64_t)(long)argc;
		cfe_ept = (long)envp;
		cfe_eptseal = (uint32_t)(unsigned long)prom_vec;
	} else {
		if ((int32_t)(long)prom_vec < 0) {
			/*
			 * Old loader; all it gives us is the handle,
			 * so use the "known" entrypoint and assume
			 * the seal.
			 */
			cfe_handle = (uint64_t)(long)prom_vec;
			cfe_ept = (uint64_t)((int32_t)0x9fc00500);
			cfe_eptseal = CFE_EPTSEAL;
		} else {
			/*
			 * Newer loaders bundle the handle/ept/eptseal
			 * Note: prom_vec is in the loader's useg
			 * which is still alive in the TLB.
			 */
			cfe_handle = (uint64_t)((int32_t *)prom_vec)[0];
			cfe_ept = (uint64_t)((int32_t *)prom_vec)[2];
			cfe_eptseal = (unsigned int)((uint32_t *)prom_vec)[3];
		}
	}
	if (cfe_eptseal != CFE_EPTSEAL) {
		/* too early for panic to do any good */
		printk("CFE's entrypoint seal doesn't match. Spinning.");
		while (1) ;
	}
	cfe_init(cfe_handle, cfe_ept);

	if (cfe_getenv("LINUX_CMDLINE", arcs_cmdline, CL_SIZE) < 0) {
        printk("Failed to get cfe_getenv\n");
    }
	if (!strstr(arcs_cmdline, "mac=")) {
        if (cfe_getenv("ETH0_HWADDR", macaddr, 32) < 0) {
            printk("Failed to get cfe_getenv\n");
        }
        else {
		    size_t clsize = strlen(arcs_cmdline);
		    strncat(arcs_cmdline, "mac=", CL_SIZE - 1 - clsize);
		    clsize = strlen(arcs_cmdline);
		    strncat(arcs_cmdline, macaddr, CL_SIZE - 1 - clsize);
        }
    }

	// Use /dev/ttyS1 as the default kernel console if none specified
	if (!strstr(arcs_cmdline, "console=")) {
		size_t clsize = strlen(arcs_cmdline);

		if (clsize && clsize < CL_SIZE - 1)
			*(arcs_cmdline + clsize++) = ' ';
		strncat(arcs_cmdline, "console=1", CL_SIZE - 1 - clsize);
	}
#ifdef CONFIG_TIVO_DEVEL
	if (!strstr(arcs_cmdline, "dsscon=")) {
		size_t clsize = strlen(arcs_cmdline);
		strncat(arcs_cmdline, " dsscon=true", CL_SIZE - 1 - clsize);
	}

	if (!strstr(arcs_cmdline, "HpkImpl=")) {
		size_t clsize = strlen(arcs_cmdline);
		strncat(arcs_cmdline, " HpkImpl=Gen06", CL_SIZE - 1 - clsize);
	}
#endif

	add_memory_region(mem_start, mem_end - mem_start, BOOT_MEM_RAM);

	tivo_generic_setup(arcs_cmdline, CL_SIZE);

	return 0;
}
#else
int __init prom_init(int argc, char **argv, char **envp, int *prom_vec)
{
	unsigned long mem_end, mem_start = 0;
	unsigned long *bblock;
	TivoConfigValue brev, brid;
	unsigned long magic = (unsigned long)envp;
	extern void tivo_generic_setup(char *, int);
        extern void (*tivo_board_reboot)(void);

	char *brev_param;

	// mips-linux expects FR bit cleared, this also makes sure the other
	// bits are in a sane state, but leave BEV on until we get our
	// exception handler installed, so that PROM gets any (unexpected)
	// exceptions
	write_32bit_cp0_register(CP0_STATUS, ST0_CU3 | ST0_BEV);
	write_32bit_cp0_register(CP0_WATCHLO, 0);
	write_32bit_cp0_register(CP0_WATCHHI, 0);

	mips_machgroup = MACH_GROUP_TIVO;

	// set this up now, in case of early panic
	tivo_board_reboot = gen06_reboot;

    if (argc != 0) {
        const char *cl = (const char *)argc;
        strncpy(arcs_cmdline, cl, CL_SIZE - 1);
    }

#ifdef CONFIG_TIVO_DEVEL
        uart_init( XTALFREQ );
        uart_puts( "Uarts Initialized.\n" );
#endif

    if (magic == 0x5469566f) {
        int fCanOverride = 0;
#ifdef CONFIG_TIVO_DEVEL
        fCanOverride = 1;
#endif
        brev = (TivoConfigValue)((unsigned long)argv & 0xffffff00);
        brid = (TivoConfigValue)((unsigned long)argv & 0x000000ff);
        if (brev == 0 || brev == 0xffffff00) {
            fCanOverride = 1;
        }
        if (fCanOverride) {
            brev_param = strstr(arcs_cmdline, "brev=");
            if (brev_param) {
                brev = simple_strtoul(brev_param + 5, NULL, 0) << 8;
            }
        }

        bblock = (unsigned long *)prom_vec;

#ifdef CONFIG_BLK_DEV_INITRD
        if (bblock[6] > bblock[5] && !(bblock[5] & 3)) {
            initrd_start = PAGE_ALIGN((unsigned long)&_end);
            initrd_end = initrd_start + bblock[6] - bblock[5];
            mem_start = virt_to_phys((void *)PAGE_ALIGN(initrd_end));
        }
#endif
    } else {
        // the console is not up yet, so no point in trying to
        // print a panic message...
        gen06_reboot();
        // not reached
        return 0;
    }

	InitTivoConfig( brev );
	SetTivoConfig( kTivoConfigBoardID, brev | brid );

	if( GetTivoConfig( kTivoConfigMemSize, &mem_end ) != 0 )
	{
	    printk( "Couldn't get the memory size configuration\n" );
	    // Use a reasonable default of 128MB
	    mem_end = 128 * 1024 * 1024;
	}

	// Use /dev/ttyS1 as the default kernel console if none specified
	if (!strstr(arcs_cmdline, "console=")) {
		size_t clsize = strlen(arcs_cmdline);

		if (clsize && clsize < CL_SIZE - 1)
			*(arcs_cmdline + clsize++) = ' ';
		strncat(arcs_cmdline, "console=1", CL_SIZE - 1 - clsize);
	}
#ifdef CONFIG_TIVO_DEVEL
	if (!strstr(arcs_cmdline, "dsscon=")) {
		size_t clsize = strlen(arcs_cmdline);
		strncat(arcs_cmdline, " dsscon=true", CL_SIZE - 1 - clsize);
	}

	if (!strstr(arcs_cmdline, "HpkImpl=")) {
		size_t clsize = strlen(arcs_cmdline);
		strncat(arcs_cmdline, " HpkImpl=Gen06", CL_SIZE - 1 - clsize);
	}
#endif

	add_memory_region(mem_start, mem_end - mem_start, BOOT_MEM_RAM);

	tivo_generic_setup(arcs_cmdline, CL_SIZE);

	return 0;
}
#endif
void __init prom_fixup_mem_map(unsigned long start, unsigned long end)
{
}

// Free the 2nd page of RAM.  The 1st is reserved for exception vectors,
// and the kernel loads starting at 0x80002000, leaving an unused page
// at 0x80001000.  The kernel cannot load at 0x80001000 because it has
// a 2-page alignment restriction, due to one of the sections being
// 2-page aligned.
//
// It would be more correct to add a linker script symbol for where the
// kernel loads, but for now we always load at 0x80002000, and the linker
// script is already complex enough...
//
void __init prom_free_prom_memory(void)
{
	unsigned long page2 = 0x80001000;
	
	ClearPageReserved(virt_to_page(page2));
	set_page_count(virt_to_page(page2), 1);
	free_page(page2);
}

int __init page_is_ram(unsigned long pagenr)
{
	return 1;
}

static int gen06_be_board_handler(struct pt_regs *regs, int is_fixup)
{
	dump_tlb_all();
	return MIPS_BE_FATAL;
}

void __init bus_error_init(void)
{
	be_board_handler = gen06_be_board_handler;
}

const char *get_system_type(void)
{
#if defined(CONFIG_TIVO_FUSION)
	return "TiVo Fusion board";
#else
	return "TiVo unknown platform";
#endif
}

