/*
 * Performance monitoring hooks
 *
 * Copyright (C) 2001, 2004 TiVo Inc. All Rights Reserved.
 */

#include <linux/ptrace.h>
#include <linux/module.h>
#include <linux/sched.h>
#include <linux/wait.h>
#include <linux/init.h>
#include <linux/plog.h>
#include <linux/sysrq.h>
#include <linux/smp_lock.h>
#include <linux/timer.h>
#include <linux/proc_fs.h>
#include <asm/uaccess.h>

/*
 * For every N ticks walk the stack of the current process
 * and dump the back trace to /var/log/kernel
 * 
 */ 
static struct timer_list tskbt_timer;
static int klogd_started = 0;
static int tskbt_rem_secs = 0;
static int tskbt_rem_count = 0;
extern struct pt_regs * show_trace_task_regs(struct task_struct *, int);
extern void ktop_start(int, int);

static void dump_tskbt_start(int secs);
static void do_dump_tsk_btrace(void);
static void dump_tsk_btrace(unsigned long);
struct proc_dir_entry * perfmon_proc_entry = NULL;
extern int do_syslog(int type, char * bug, int count);
static int perfmon_proc_init(void);
#if 0
static void perfmon_proc_cleanup(void);
#endif
static int perfmon_proc_read(char * buffer, char ** start, off_t offset,
				int size, int * eof, void * data);
static int perfmon_proc_write(struct file * filp, const char * buffer,
				 unsigned long count, void * data);

static int perfmon_proc_read(char * buffer, char ** start, off_t offset,
				int size, int * eof, void * data)
{
	int len = 0;
	len += sprintf(buffer,"%d\n",tskbt_rem_secs);
	*eof = 1;
	return len;
}

static int perfmon_proc_write(struct file * filp, const char * buffer,
				 unsigned long count, void * data)
{
	int num;
	int run_seconds;
	char buff[11];

	if ( tskbt_rem_count > 0 )
		return -EINVAL;
	if (!capable(CAP_SYS_ADMIN) )
		return -EACCES;
	num = (count < 10) ? count : 10;
	copy_from_user(buff, buffer, num);
	buff[num] = '\0';
	run_seconds = simple_strtoul(buff, 0, 10);
	run_seconds = (run_seconds < 0) ? 0 : run_seconds;
	dump_tskbt_start(run_seconds);
	ktop_start(run_seconds, 0);
	return count;
}

static int perfmon_proc_init(void)
{
	if ( perfmon_proc_entry )
		return 1; 
	perfmon_proc_entry  = create_proc_entry("perfmon", S_IRUGO | S_IWUSR, NULL);
	if (perfmon_proc_entry) {
		perfmon_proc_entry->read_proc =  perfmon_proc_read;
		perfmon_proc_entry->write_proc = perfmon_proc_write;
	} else {
		printk("Unable to create proc entry for perfmon!\n");
		return 1;
	}
	return 0;
}

#if 0
static void perfmon_proc_cleanup(void)
{
	if (! perfmon_proc_entry)
		return;
	remove_proc_entry("perfmon", perfmon_proc_entry);
}
#endif

static int __init perfmon_init(void)
{
	perfmon_proc_init();
	return 0;
}

static void dump_tskbt_start(int secs) {
	init_timer(&tskbt_timer);  /* init the timer structure */
	tskbt_timer.function = dump_tsk_btrace;
	tskbt_timer.data = (unsigned long)0;
	tskbt_rem_secs=secs;
	tskbt_rem_count=secs * 10;
	mod_timer(&tskbt_timer, jiffies + HZ/10);
}

static void dump_tsk_btrace(unsigned long dummy)
{
	(void) dummy;
	struct task_struct *tsk;
    
	if ( klogd_started && (tskbt_rem_count > 0)) {
		do_dump_tsk_btrace();
		tskbt_rem_count--;
		if ( tskbt_rem_count < 0 ) tskbt_rem_count = 0;
		tskbt_rem_secs = (tskbt_rem_count / 10);
	} else if (tskbt_rem_count <= 0) {
		/*
		 * In the case that we're not supposed to
		 * run anymore, we don't want the fall
		 * through case which re-activates the
		 * timer
		 *
		 */
		return;
	} else {
		/* FIXME Discuss need for klogd detector */
		for_each_task(tsk) {
			if ( !strcmp (tsk->comm,"klogd")){
				/*Dont dump until klogd has been started*/
				klogd_started++;
				break;
			}
		}
	}

	mod_timer(&tskbt_timer,jiffies + HZ/10);
}

static void do_dump_tsk_btrace(void)
{
	printk(KERN_INFO "PID %d (%s):\n", current->pid, current->comm);
	show_trace_task_regs(current, 0);
}
__initcall(perfmon_init);
